import { ItemComponentTypes, system, world } from "@minecraft/server";
import { tryVillageWaystoneGeneration } from "./worldgen";
import { destroyWaystoneBase, interactWaystone, WaystoneComponent, WaystoneTopComponent } from "./waystones";
import { AttunedShardComponent, DormantShardComponent, removeWarpPlateAt, sneakHijackInteractWarpPlate, WarpPlateAttunedComponent, WarpPlateAttuningComponent, WarpPlateEmptyComponent, WarpPlateWarpingComponent, } from "./warp_plates";
import { BlankScrollComponent, BoundScrollComponent, PortalScrollComponent, ReturnScrollComponent, WarpScrollComponent, WarpStoneComponent, } from "./scrolls";
import { destroySharestoneBase, interactSharestone, removeSharestoneAt, SharestoneComponent, SharestoneTopComponent, } from "./sharestones";
import { destroyPortstoneBase, PortstoneComponent, PortstoneTopComponent } from "./portstones";
import { getWaystoneForBlock, isSharestoneType, removeWaystoneAt, WaystoneType } from "./waystone_registry";
import { showWaystoneSetup, showSharestoneSetup } from "./ui/ui_waystone_setup";
import { GuideBookComponent, TomeOfSecrecyComponent } from "./tomes";
import { WarpPortalComponent, WarpPortalEmitterComponent } from "./warp_portals";
import { spawnGuideBook } from "./guide_book";
world.beforeEvents.worldInitialize.subscribe((initEvent) => {
    initEvent.blockComponentRegistry.registerCustomComponent("blay_waystones:waystone", new WaystoneComponent());
    initEvent.blockComponentRegistry.registerCustomComponent("blay_waystones:waystone_top", new WaystoneTopComponent());
    initEvent.blockComponentRegistry.registerCustomComponent("blay_waystones:sharestone", new SharestoneComponent());
    initEvent.blockComponentRegistry.registerCustomComponent("blay_waystones:sharestone_top", new SharestoneTopComponent());
    initEvent.blockComponentRegistry.registerCustomComponent("blay_waystones:portstone", new PortstoneComponent());
    initEvent.blockComponentRegistry.registerCustomComponent("blay_waystones:portstone_top", new PortstoneTopComponent());
    initEvent.blockComponentRegistry.registerCustomComponent("blay_waystones:warp_plate_empty", new WarpPlateEmptyComponent());
    initEvent.blockComponentRegistry.registerCustomComponent("blay_waystones:warp_plate_attuning", new WarpPlateAttuningComponent());
    initEvent.blockComponentRegistry.registerCustomComponent("blay_waystones:warp_plate_attuned", new WarpPlateAttunedComponent());
    initEvent.blockComponentRegistry.registerCustomComponent("blay_waystones:warp_plate_warping", new WarpPlateWarpingComponent());
    initEvent.blockComponentRegistry.registerCustomComponent("blay_waystones:warp_portal", new WarpPortalComponent());
    initEvent.blockComponentRegistry.registerCustomComponent("blay_waystones:warp_portal_emitter", new WarpPortalEmitterComponent());
    initEvent.itemComponentRegistry.registerCustomComponent("blay_waystones:dormant_shard", new DormantShardComponent());
    initEvent.itemComponentRegistry.registerCustomComponent("blay_waystones:attuned_shard", new AttunedShardComponent());
    initEvent.itemComponentRegistry.registerCustomComponent("blay_waystones:blank_scroll", new BlankScrollComponent());
    initEvent.itemComponentRegistry.registerCustomComponent("blay_waystones:bound_scroll", new BoundScrollComponent());
    initEvent.itemComponentRegistry.registerCustomComponent("blay_waystones:return_scroll", new ReturnScrollComponent());
    initEvent.itemComponentRegistry.registerCustomComponent("blay_waystones:warp_scroll", new WarpScrollComponent());
    initEvent.itemComponentRegistry.registerCustomComponent("blay_waystones:portal_scroll", new PortalScrollComponent());
    initEvent.itemComponentRegistry.registerCustomComponent("blay_waystones:warp_stone", new WarpStoneComponent());
    initEvent.itemComponentRegistry.registerCustomComponent("blay_waystones:tome_of_secrecy", new TomeOfSecrecyComponent());
    initEvent.itemComponentRegistry.registerCustomComponent("blay_waystones:guide_book", new GuideBookComponent());
});
world.beforeEvents.itemUseOn.subscribe((event) => {
    if (event.source.isSneaking) {
        // We hijack only items with durability (e.g. tools) and food. Block placements against waystones while sneaking should still be possible uninterrupted.
        const shouldHijackItem = event.itemStack.hasComponent(ItemComponentTypes.Durability) ||
            event.itemStack.hasComponent(ItemComponentTypes.Food) ||
            event.itemStack.hasTag("blay_waystones:hijacks_sneak_use");
        const shouldHijackBlock = event.block.hasTag("blay_waystones:hijacks_sneak_use");
        // itemUseOn before is spammed, so we add a cooldown here
        const isOnCooldown = event.source.getItemCooldown("blay_waystones:sneak_use") > 0;
        if (shouldHijackItem && shouldHijackBlock && !isOnCooldown) {
            let effectiveBlock = event.block;
            if (effectiveBlock.hasTag("blay_waystones:dummy_top")) {
                effectiveBlock = effectiveBlock.below();
            }
            if (effectiveBlock) {
                const waystone = getWaystoneForBlock(event.block);
                if (waystone) {
                    const waystoneType = waystone?.data.type;
                    if (waystoneType === WaystoneType.Waystone) {
                        system.run(() => {
                            if (event.source.getItemCooldown("blay_waystones:sneak_use") <= 0) {
                                event.source.startItemCooldown("blay_waystones:sneak_use", 20);
                                interactWaystone(event.source, waystone);
                            }
                        });
                        event.cancel = true;
                    }
                    else if (waystoneType && isSharestoneType(waystoneType)) {
                        system.run(() => {
                            if (event.source.getItemCooldown("blay_waystones:sneak_use") <= 0) {
                                event.source.startItemCooldown("blay_waystones:sneak_use", 20);
                                interactSharestone(event.source, waystoneType, waystone);
                            }
                        });
                        event.cancel = true;
                    }
                    else if (waystoneType == WaystoneType.WarpPlate) {
                        system.run(() => {
                            if (event.source.getItemCooldown("blay_waystones:sneak_use") <= 0) {
                                event.source.startItemCooldown("blay_waystones:sneak_use", 20);
                                sneakHijackInteractWarpPlate(event.source, event.block);
                            }
                        });
                        event.cancel = true;
                    }
                }
            }
        }
    }
});
world.afterEvents.playerPlaceBlock.subscribe((event) => {
    // Place event in components does not have a player reference, so we have to do this with a global event handler still.
    if (event.block.hasTag("blay_waystones:waystone")) {
        const waystone = getWaystoneForBlock(event.block);
        if (waystone) {
            showWaystoneSetup(event.player, waystone);
        }
    }
    else if (event.block.hasTag("blay_waystones:sharestone")) {
        const waystone = getWaystoneForBlock(event.block);
        if (waystone) {
            showSharestoneSetup(event.player, waystone);
        }
    }
});
world.afterEvents.blockExplode.subscribe((event) => {
    // Explode is not part of block components, so we have to do this with a global event handler still.
    if (event.explodedBlockPermutation.hasTag("blay_waystones:waystone")) {
        if (event.explodedBlockPermutation.getState("blay_waystones:half") === "top") {
            destroyWaystoneBase(event.dimension, event.block.below());
        }
        else {
            removeWaystoneAt(event.dimension.id, event.block.location);
        }
    }
    else if (event.explodedBlockPermutation.hasTag("blay_waystones:sharestone")) {
        if (event.explodedBlockPermutation.getState("blay_waystones:half") === "top") {
            destroySharestoneBase(event.dimension, event.block.below());
        }
        else {
            removeSharestoneAt(event.dimension.id, event.block.location);
        }
    }
    else if (event.explodedBlockPermutation.hasTag("blay_waystones:warp_plate")) {
        removeWarpPlateAt(event.dimension.id, event.block.location);
    }
    else if (event.explodedBlockPermutation.hasTag("blay_waystones:portstone")) {
        if (event.explodedBlockPermutation.getState("blay_waystones:half") === "top") {
            destroyPortstoneBase(event.dimension, event.block.below());
        }
    }
});
world.afterEvents.entitySpawn.subscribe((event) => {
    // Cats spawn in larger villages, so we use them to generate our waystone structure
    if (event.cause === "Event" && event.entity.isValid() && event.entity.typeId === "minecraft:cat") {
        tryVillageWaystoneGeneration(event.entity.dimension, event.entity.location);
    }
});
world.afterEvents.playerSpawn.subscribe((event) => {
    // Spawn a tutorial book when the player first joins
    const tutorialBookGrantedProperty = "blay_waystones:tutorial_book";
    if (!event.player.getDynamicProperty(tutorialBookGrantedProperty)) {
        spawnGuideBook(event.player);
        event.player.setDynamicProperty(tutorialBookGrantedProperty, true);
    }
});

//# sourceMappingURL=../../_blay_waystonesDebug/main.js.map
