import { WaystoneType } from "./waystone_registry";
import { teleportToWaystone } from "./teleporter";
import { showSharestoneSelection, showWaystoneSelection } from "./ui/ui_waystone_selection";
import { getActivatedWaystones } from "./player_persistence";
import { getSharestones } from "./sharestones";
import { playRegularTeleportEffects } from "./teleport_effects";
export class PortstoneComponent {
    beforeOnPlayerPlace(event) {
        // Only allow placement if the block above is free
        const above = event.block.above();
        if (!above?.isAir) {
            event.cancel = true;
        }
    }
    onPlace(event) {
        const above = event.block.above();
        if (above?.isAir) {
            above.setPermutation(event.block.permutation.withState("blay_waystones:half", "top"));
        }
    }
    onPlayerInteract(event) {
        const player = event.player;
        const targetType = getPortstoneTargetType(event.block);
        if (player && targetType) {
            interact(player, targetType);
        }
    }
}
export class PortstoneTopComponent {
    onPlayerInteract(event) {
        const player = event.player;
        if (player) {
            let baseBlock = event.block?.below();
            const targetType = getPortstoneTargetType(baseBlock);
            if (player && targetType) {
                interact(player, targetType);
            }
        }
    }
    onPlayerDestroy(event) {
        destroyPortstoneBase(event.dimension, event.block.below());
    }
}
const mappings = new Map([
    ["blay_waystones:portstone", WaystoneType.Waystone],
    ["blay_waystones:amethyst_portstone", WaystoneType.AmethystSharestone],
    ["blay_waystones:copper_portstone", WaystoneType.CopperSharestone],
    ["blay_waystones:diamond_portstone", WaystoneType.DiamondSharestone],
    ["blay_waystones:emerald_portstone", WaystoneType.EmeraldSharestone],
    ["blay_waystones:gold_portstone", WaystoneType.GoldSharestone],
    ["blay_waystones:lapis_portstone", WaystoneType.LapisSharestone],
    ["blay_waystones:prismarine_portstone", WaystoneType.PrismarineSharestone],
    ["blay_waystones:redstone_portstone", WaystoneType.RedstoneSharestone],
]);
export function getPortstoneTargetType(block) {
    if (!block) {
        return undefined;
    }
    for (const [permutation, type] of mappings) {
        if (block.permutation.matches(permutation)) {
            return type;
        }
    }
}
async function interact(player, targetType) {
    let selectedWaystone;
    if (targetType === WaystoneType.Waystone) {
        selectedWaystone = await showWaystoneSelection(player, getActivatedWaystones(player));
    }
    else if (targetType) {
        selectedWaystone = await showSharestoneSelection(player, getSharestones(targetType));
    }
    if (selectedWaystone && player.isValid()) {
        teleportToWaystone(player, selectedWaystone);
        playRegularTeleportEffects(player, selectedWaystone);
    }
}
export function destroyPortstoneBase(dimension, block) {
    if (block?.hasTag("blay_waystones:portstone")) {
        dimension.runCommand("setblock " + block.x + " " + block.y + " " + block.z + " air destroy");
    }
}

//# sourceMappingURL=../../_blay_waystonesDebug/portstones.js.map
