import { EntityComponentTypes, world, } from "@minecraft/server";
import { getDirectionOffset, vectorAdd, vectorCenter } from "./utils";
import { verifyWaystoneBlock, WaystoneType } from "./waystone_registry";
import { setWarpPortalEntityCooldown, verifyWarpPortal } from "./warp_portals";
export function teleportToWaystone(entity, waystone, options) {
    const isInterdimensional = entity.dimension.id !== waystone.dimensionId;
    if (!isInterdimensional && (options?.includeLeashed ?? true)) {
        const nearbyEntities = entity.dimension.getEntities({
            location: entity.location,
            maxDistance: 10,
        });
        for (const nearbyEntity of nearbyEntities) {
            const leashable = nearbyEntity.getComponent(EntityComponentTypes.Leashable);
            if (leashable) {
                if (leashable.isLeashed && leashable.leashHolder?.id === entity.id) {
                    teleportToWaystone(nearbyEntity, waystone, {
                        includeLeashed: false,
                        includeMount: true,
                    });
                }
            }
        }
    }
    if (!isInterdimensional && (options?.includeMount ?? true)) {
        const riding = entity.getComponent(EntityComponentTypes.Riding);
        if (riding?.entityRidingOn) {
            teleportToWaystone(riding.entityRidingOn, waystone, options);
            // Normally we would want to early-return here since the mount should carry the passenger along,
            // but unfortunately it seems to break over bigger distances, leaving the horse somewhere in the middle between the waystones
            // Therefore, we teleport rider and mount individually, which will dismount.
            // Remounting carries the same risk of the horse ending up somewhere mid-teleport, so we don't do that either
        }
    }
    const offset = getDirectionOffset(waystone.data.facing);
    const targetPos = vectorAdd(vectorCenter(waystone.pos), offset);
    const targetDimension = entity.dimension.id !== waystone.dimensionId ? world.getDimension(waystone.dimensionId) : undefined;
    if (waystone.id === "return_portal") {
        setWarpPortalEntityCooldown(entity, waystone);
    }
    entity.teleport(targetPos, {
        facingLocation: vectorAdd(targetPos, offset),
        dimension: targetDimension,
    });
    if (waystone.data.type === WaystoneType.WarpPortal && entity.typeId === "minecraft:player") {
        verifyWarpPortal(entity, waystone);
    }
    else {
        verifyWaystoneBlock(waystone);
    }
}

//# sourceMappingURL=../../_blay_waystonesDebug/teleporter.js.map
