import { ActionFormData } from "@minecraft/server-ui";
import { addPlayerToWaystoneTeam, getPlayerActivatedWaystoneIds, getPlayerWaystoneTeams, getTeamActivatedWaystoneIds, initializeDefaultTeam, removePlayerFromWaystoneTeam, } from "../player_persistence";
import { getDefaultTeam } from "../config";
export async function showTomeOfSecrecy(player) {
    const defaultTeam = getDefaultTeam();
    if (!defaultTeam) {
        return;
    }
    initializeDefaultTeam(player);
    const alreadyRead = getPlayerWaystoneTeams(player).indexOf(getDefaultTeam()) === -1;
    const playerWaystones = getPlayerActivatedWaystoneIds(player);
    const teamWaystones = getTeamActivatedWaystoneIds(defaultTeam);
    const teamExclusiveWaystones = teamWaystones.filter((id) => playerWaystones.indexOf(id) === -1);
    if (alreadyRead) {
        const form = new ActionFormData()
            .title({ translate: "gui.blay_waystones.tome_of_secrecy_read.title" })
            .body({
            rawtext: [
                {
                    translate: "gui.blay_waystones.tome_of_secrecy_read.body.1",
                },
                { text: "\n\n" },
                {
                    translate: "gui.blay_waystones.tome_of_secrecy_read.body.2",
                },
                { text: "\n\n" },
                teamExclusiveWaystones.length
                    ? {
                        translate: "gui.blay_waystones.tome_of_secrecy_read.body.3",
                        with: [String(teamExclusiveWaystones.length)],
                    }
                    : {
                        translate: "gui.blay_waystones.tome_of_secrecy_read.body.3a",
                    },
            ],
        })
            .button({ translate: "gui.blay_waystones.tome_of_secrecy_read.confirm" })
            .button({ translate: "gui.blay_waystones.tome_of_secrecy_read.cancel" });
        const response = await form.show(player);
        if (response.canceled || response.selection === undefined || !player.isValid()) {
            return;
        }
        if (response.selection === 0) {
            addPlayerToWaystoneTeam(player, defaultTeam);
            player.sendMessage({ translate: "chat.blay_waystones.tome_of_secrecy.forgot" });
        }
    }
    else {
        const form = new ActionFormData()
            .title({ translate: "gui.blay_waystones.tome_of_secrecy.title" })
            .body({
            rawtext: [
                {
                    translate: "gui.blay_waystones.tome_of_secrecy.body.1",
                },
                { text: "\n\n" },
                teamExclusiveWaystones.length
                    ? {
                        translate: "gui.blay_waystones.tome_of_secrecy.body.2",
                        with: [String(teamExclusiveWaystones.length)],
                    }
                    : {
                        translate: "gui.blay_waystones.tome_of_secrecy.body.2a",
                    },
                { text: "\n\n" },
                {
                    translate: "gui.blay_waystones.tome_of_secrecy.body.3",
                },
            ],
        })
            .button({ translate: "gui.blay_waystones.tome_of_secrecy.confirm" })
            .button({ translate: "gui.blay_waystones.tome_of_secrecy.cancel" });
        const response = await form.show(player);
        if (response.canceled || response.selection === undefined || !player.isValid()) {
            return;
        }
        if (response.selection === 0) {
            removePlayerFromWaystoneTeam(player, defaultTeam);
            player.sendMessage({ translate: "chat.blay_waystones.tome_of_secrecy.read" });
        }
    }
}

//# sourceMappingURL=../../../_blay_waystonesDebug/ui/ui_tome_of_secrecy.js.map
