import { ModalFormData } from "@minecraft/server-ui";
import { getWaystoneForPos, saveWaystoneData } from "../waystone_registry";
import { isWaystoneFavorite, saveWaystoneActivation, setWaystoneFavorite } from "../player_persistence";
import { generateName } from "../name_generator";
import { GROUP_PLAYER_HOMES, groupsWithoutFavorite } from "../waystone_groups";
import { saveSharestoneData } from "../sharestones";
export async function showWaystoneSetup(player, waystone) {
    showGenericWaystoneSetup(player, waystone, {
        title: { translate: "gui.blay_waystones.waystone_setup.title" },
        cancelHandler: () => {
            const existing = getWaystoneForPos(waystone.dimensionId, waystone.pos);
            waystone = existing ?? waystone;
            if (!waystone.data.name) {
                waystone.data.name = generateName();
            }
            if (!waystone.data.group) {
                waystone.data.group = GROUP_PLAYER_HOMES;
            }
            saveWaystoneData(waystone);
            saveWaystoneActivation(player, waystone);
        },
        submitHandler: (values) => {
            const existing = getWaystoneForPos(waystone.dimensionId, waystone.pos);
            waystone = existing ?? waystone;
            waystone.data.name = values.name;
            if (!waystone.data.name?.trim()) {
                waystone.data.name = generateName();
            }
            waystone.data.group = values.group;
            setWaystoneFavorite(player, waystone, values.favorite);
            saveWaystoneData(waystone);
            saveWaystoneActivation(player, waystone);
        },
    });
}
export async function showSharestoneSetup(player, waystone) {
    showGenericWaystoneSetup(player, waystone, {
        title: { translate: "gui.blay_waystones.sharestone_setup.title" },
        cancelHandler: () => {
            const existing = getWaystoneForPos(waystone.dimensionId, waystone.pos);
            waystone = existing ?? waystone;
            if (!waystone.data.name) {
                waystone.data.name = generateName();
            }
            if (!waystone.data.group) {
                waystone.data.group = GROUP_PLAYER_HOMES;
            }
            saveSharestoneData(waystone);
        },
        submitHandler: (values) => {
            const existing = getWaystoneForPos(waystone.dimensionId, waystone.pos);
            waystone = existing ?? waystone;
            waystone.data.name = values.name;
            if (!waystone.data.name?.trim()) {
                waystone.data.name = generateName();
            }
            waystone.data.group = values.group;
            setWaystoneFavorite(player, waystone, values.favorite);
            saveSharestoneData(waystone);
        },
    });
}
async function showGenericWaystoneSetup(player, waystone, options) {
    const modalForm = new ModalFormData().title(options?.title ?? { translate: "gui.blay_waystones.waystone_setup.title" });
    modalForm.textField({ translate: "gui.blay_waystones.waystone_setup.name" }, { translate: "gui.blay_waystones.waystone_setup.placeholder" }, waystone.data.name);
    let prevGroupIndex = groupsWithoutFavorite.findIndex((group) => group.id === waystone.data.group);
    if (prevGroupIndex === -1) {
        prevGroupIndex = 0;
    }
    modalForm.dropdown({ translate: "gui.blay_waystones.waystone_setup.group" }, groupsWithoutFavorite.map((group) => group.label), prevGroupIndex);
    const prevFavorite = isWaystoneFavorite(player, waystone);
    modalForm.toggle({
        translate: "gui.blay_waystones.waystone_setup.favorite",
    }, prevFavorite);
    try {
        const formData = await modalForm.show(player);
        if (formData.canceled || !player.isValid()) {
            options?.cancelHandler();
            return;
        }
        let name = formData.formValues?.[0];
        if (name?.length > 32) {
            name = name.substring(0, 32);
        }
        const favorite = formData.formValues?.[2];
        const groupIndex = formData.formValues?.[1];
        const group = groupsWithoutFavorite[groupIndex];
        options?.submitHandler({
            name,
            favorite,
            group: group?.id,
        });
    }
    catch (error) {
        player.sendMessage("Failed to show form: " + error);
    }
}

//# sourceMappingURL=../../../_blay_waystonesDebug/ui/ui_waystone_setup.js.map
