import { EntityComponentTypes, GameMode, ItemComponentTypes } from "@minecraft/server";
import { Direction, EquipmentSlot } from "@minecraft/server";
export function randomUUID() {
    return "xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx".replace(/[xy]/g, function (c) {
        var r = (Math.random() * 16) | 0, v = c === "x" ? r : (r & 0x3) | 0x8;
        return v.toString(16);
    });
}
export function parseDirection(direction) {
    switch (direction?.toLowerCase()) {
        case "north":
            return Direction.North;
        case "south":
            return Direction.South;
        case "west":
            return Direction.West;
        case "east":
            return Direction.East;
        default:
            return undefined;
    }
}
export function vectorAdd(pos, offset) {
    return {
        x: pos.x + offset.x,
        y: pos.y + offset.y,
        z: pos.z + offset.z,
    };
}
export function vectorSubtract(a, b) {
    return {
        x: a.x - b.x,
        y: a.y - b.y,
        z: a.z - b.z,
    };
}
export function vectorNormalize(pos) {
    const length = Math.sqrt(pos.x * pos.x + pos.y * pos.y + pos.z * pos.z);
    return {
        x: pos.x / length,
        y: pos.y / length,
        z: pos.z / length,
    };
}
export function vectorCenterH(pos) {
    return {
        x: pos.x + 0.5,
        y: pos.y,
        z: pos.z + 0.5,
    };
}
export function vectorCenter(pos) {
    return {
        x: pos.x + 0.5,
        y: pos.y + 0.5,
        z: pos.z + 0.5,
    };
}
export function vectorRotateY(pos, angle) {
    const cos = Math.cos(angle);
    const sin = Math.sin(angle);
    return {
        x: pos.x * cos - pos.z * sin,
        y: pos.y,
        z: pos.x * sin + pos.z * cos,
    };
}
export function getDirectionOffset(direction) {
    switch (direction) {
        case Direction.North:
            return { x: 0, y: 0, z: -1 };
        case Direction.South:
            return { x: 0, y: 0, z: 1 };
        case Direction.West:
            return { x: -1, y: 0, z: 0 };
        case Direction.East:
            return { x: 1, y: 0, z: 0 };
        case Direction.Up:
            return { x: 0, y: 1, z: 0 };
        case Direction.Down:
            return { x: 0, y: -1, z: 0 };
        default:
            return { x: 0, y: 0, z: 0 };
    }
}
export function getDirectionFromVectorH(dir) {
    if (Math.abs(dir.x) > Math.abs(dir.z)) {
        if (dir.x > 0) {
            return Direction.East;
        }
        else {
            return Direction.West;
        }
    }
    else {
        if (dir.z > 0) {
            return Direction.South;
        }
        else {
            return Direction.North;
        }
    }
}
export function removeItemFromHand(entity, item, amount = 1) {
    if (entity.typeId === "minecraft:player" && entity.getGameMode() === GameMode.creative) {
        return;
    }
    const equippable = entity.getComponent(EntityComponentTypes.Equippable);
    if (!equippable) {
        return;
    }
    const handItem = equippable.getEquipment(EquipmentSlot.Mainhand);
    const offhandItem = equippable.getEquipment(EquipmentSlot.Offhand);
    if (handItem?.typeId === item.typeId) {
        if (handItem.amount <= amount) {
            equippable.setEquipment(EquipmentSlot.Mainhand, undefined);
        }
        else {
            handItem.amount -= amount;
            equippable.setEquipment(EquipmentSlot.Mainhand, handItem.amount > 0 ? handItem : undefined);
        }
    }
    else if (offhandItem?.typeId === item.typeId) {
        if (offhandItem.amount <= amount) {
            equippable.setEquipment(EquipmentSlot.Offhand, undefined);
        }
        else {
            offhandItem.amount -= amount;
            equippable.setEquipment(EquipmentSlot.Offhand, offhandItem.amount > 0 ? offhandItem : undefined);
        }
    }
}
export function damageItemInHand(entity, item, amount = 1) {
    if (entity.typeId === "minecraft:player" && entity.getGameMode() === GameMode.creative) {
        return;
    }
    const equippable = entity.getComponent(EntityComponentTypes.Equippable);
    if (!equippable) {
        return;
    }
    const handItem = equippable.getEquipment(EquipmentSlot.Mainhand);
    const offhandItem = equippable.getEquipment(EquipmentSlot.Offhand);
    if (handItem?.typeId === item.typeId) {
        damageItemInSpecificHand(entity, handItem, EquipmentSlot.Mainhand, amount);
    }
    else if (offhandItem?.typeId === item.typeId) {
        damageItemInSpecificHand(entity, offhandItem, EquipmentSlot.Offhand, amount);
    }
}
export function damageItemInSpecificHand(entity, item, hand, amount = 1) {
    const equippable = entity.getComponent(EntityComponentTypes.Equippable);
    if (!equippable) {
        return;
    }
    const durability = item.getComponent(ItemComponentTypes.Durability);
    if (!durability) {
        return;
    }
    if (Math.round(Math.random() * 100) > durability.getDamageChance(0)) {
        return;
    }
    if (durability.damage + amount >= durability.maxDurability) {
        equippable.setEquipment(hand, undefined);
    }
    else {
        durability.damage += amount;
        equippable.setEquipment(hand, item);
    }
}
export function getDistanceH(a, b) {
    return Math.sqrt(Math.pow(a.x - b.x, 2) + Math.pow(a.z - b.z, 2));
}
export function getDistance(a, b) {
    return Math.sqrt(Math.pow(a.x - b.x, 2) + Math.pow(a.y - b.y, 2) + Math.pow(a.z - b.z, 2));
}
export function returnItemToPlayer(entity, itemStack) {
    const equippable = entity.getComponent(EntityComponentTypes.Equippable);
    if (equippable && !equippable.getEquipment(EquipmentSlot.Mainhand)) {
        equippable.setEquipment(EquipmentSlot.Mainhand, itemStack);
    }
    else {
        const inventory = entity.getComponent(EntityComponentTypes.Inventory);
        if (inventory) {
            const rest = inventory.container?.addItem(itemStack);
            if (rest) {
                entity.dimension.spawnItem(rest, entity.location);
            }
        }
        else {
            entity.dimension.spawnItem(itemStack, entity.location);
        }
    }
}
export function regexEscape(input) {
    return input.replace(/[-[\]{}()*+?.,\\^$|#\s]/g, "\\$&");
}
export function isAirOrReplaceable(block) {
    if (!block || block.isAir) {
        return true;
    }
    const replaceableBlocks = [
        "minecraft:short_grass",
        "minecraft:tall_grass",
        "minecraft:red_flower",
        "minecraft:yellow_flower",
        "minecraft:double_plant",
        "minecraft:deadbush",
        "minecraft:snow_layer",
    ];
    for (const replaceableBlock of replaceableBlocks) {
        if (block.matches(replaceableBlock)) {
            return true;
        }
    }
    return false;
}
export function isTeleportAllowed(entity, boundTo) {
    if (entity.typeId === "minecraft:player") {
        return true;
    }
    else {
        // There is a bug on Realms where teleporting an entity or item to another dimension using the API, it will DELETE the entity if the dimension is not loaded.
        // Therefore, we unfortunately have to block interdimensional teleport of items and entities.
        return entity.dimension.id === boundTo.dimensionId;
    }
}

//# sourceMappingURL=../../_blay_waystonesDebug/utils.js.map
